from typing import OrderedDict
import chromeRequests
import requests
import time

session = chromeRequests.session()

response = session.get("https://www.google.com/", headers=OrderedDict(
    {"User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/80.0.3987.132 Safari/537.36"}))
print(response.status_code)



import json
from typing import OrderedDict
from .response import Response
from .lib import loadLibrary
from .headers import Headers
from .cookies import Cookies
from .utils import *

request = None

library = loadLibrary()


class Session:
    def __init__(self, proxy="", one_time=False, library=None):
        if library is None:
            raise Exception("Library is not loaded")
        self.__session = library.createSession
        self.__session.restype = ctypes.c_void_p
        self.__request = library.request
        self.__request.restype = ctypes.c_void_p
        self.__changeProxy = library.changeProxy
        self.__changeProxy.restype = ctypes.c_void_p
        self.__uuid = json.loads(pull_from_mem(self.__session(
            proxy.encode('utf-8'))) if not one_time else "")['sessionId']
        self.__closeSession = library.closeSession
        self.__closeSession.restype = ctypes.c_void_p
        self.cookies = Cookies()
        self.headers = Headers()

    def set_proxy(self, proxy: str):
        payload = {
            "session": self.__uuid,
            "proxy": proxy,
        }
        load = json.dumps(payload, sort_keys=True).encode('utf-8')
        self.__changeProxy(load)

    def request(self, load):
        response = self.__request(load)
        json_ = json.loads(pull_from_mem(response))
        check_error(json_)
        toReturn = Response(json_)
        cookies = toReturn.cookies
        self.cookies.update(cookies)
        return toReturn

    def createPayload(self, request_type: str, url: str, **kwargs) -> dict:
        payload = OrderedDict({
            "session": self.__uuid,
            "requestType": request_type,
            "url": url,
            "parameters": {
                "url": url,
            }
        })
        allowed_get_params = ['proxy', 'headers', "cookies", "allow_redirects"]
        allowed_post_params = ['json', 'proxy',
                               'headers', 'data', "cookies", "allow_redirects"]
        params = allowed_get_params if request_type == "GET" else allowed_post_params
        for item in kwargs:
            if not item in params:
                raise Exception(f"{item} is not an accepted PARAM")
        payload['parameters']['headers'] = kwargs.get("headers", {})
        keys = list(payload['parameters']['headers'].keys())
        payload['parameters']['headerOrder'] = keys
        payload['parameters']['headers'].update(self.headers.get_dict())
        payload['parameters']['proxy'] = kwargs.get("proxy", "")
        payload['parameters']['cookies'] = kwargs.get("cookies", {})
        payload['parameters']['redirects'] = kwargs.get(
            "allow_redirects", True)
        payload['parameters']['cookies'].update(self.cookies.get_dict())
        if request_type == "POST" or request_type == "PUT":
            payload['parameters']['form'] = kwargs.get("data", [])
            payload['parameters']['json'] = json.dumps(
                kwargs.get("json", {}), sort_keys=True)
        return payload

    def get(self, url: str, **kwargs) -> Response:
        payload = self.createPayload("GET", url, **kwargs)
        toSend = json.dumps(
            payload, sort_keys=True).encode('utf-8')
        response = self.request(toSend)
        return response

    def post(self, url: str, **kwargs) -> Response:
        payload = self.createPayload("POST", url, **kwargs)
        response = self.request(json.dumps(
            payload, sort_keys=True).encode('utf-8'))
        return response

    def put(self, url: str, **kwargs) -> Response:
        payload = self.createPayload("PUT", url, **kwargs)
        response = self.request(json.dumps(
            payload, sort_keys=True).encode('utf-8'))
        return response

    def close(self):
        self.__closeSession(self.__uuid.encode('utf-8'))

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        if self.__uuid != "":
            self.close()
        return False



from .lib import loadLibrary
from .response import Response
from . import sessions
import ctypes
import json

globalLibrary = loadLibrary()


def get(url: str, **kwargs) -> Response:
    with sessions.Session(one_time=True, library=globalLibrary) as session:
        return session.get(url, **kwargs)


def post(url: str, **kwargs) -> Response:
    with sessions.Session(one_time=True, library=globalLibrary) as session:
        return session.post(url, **kwargs)


def put(url: str, **kwargs) -> Response:
    with sessions.Session(one_time=True, library=globalLibrary) as session:
        return session.put(url, **kwargs)


def session(proxy="") -> sessions.Session:

    return sessions.Session(proxy, library=globalLibrary)



class Cookies:
    def __init__(self):
        self.__cookies = {}

    def set(self, name: str, value: str) -> None:
        self.__cookies.update({
            name: value,
        })

    def update(self, cookies: dict):
        self.__cookies.update(cookies)

    def get_dict(self) -> dict:
        return self.__cookies

    def clear(self) -> None:
        self.__cookies = {}

    def __str__(self) -> str:
        return str(self.__cookies)



from typing import OrderedDict


class Headers:
    def __init__(self):
        self.__headers = OrderedDict()

    def update(self, headers: dict) -> None:
        self.__headers.update(headers)

    def get_dict(self) -> dict:
        ordered = OrderedDict()
        for key in self.__headers:
            ordered[key.lower()] = self.__headers[key]
        return ordered

    def clear(self) -> None:
        self.__headers = {}

    def __str__(self) -> str:
        return str(self.__headers)



from glob import glob
from json import load
import subprocess
import ctypes
import os


def checkExistanceOfGo():
    try:
        subprocess.check_output(["go", "version"])
        return True
    except:
        return False


def compileLibrary():
    try:
        currentDir = os.getcwd()
        os.chdir(os.path.abspath(os.path.join(
            os.path.dirname(__file__), '')) + "/go")
        subprocess.check_output(
            ["go", "build", "-buildmode=c-shared", "-o", "./compiled/library.so"])
        os.chdir(currentDir)
        return True
    except:
        return False


def loadLibrary():
    currentDir = (os.path.abspath(
        os.path.join(os.path.dirname(__file__),)))
    if not os.path.exists(currentDir + "/go/compiled/library.so"):
        print("Library not found, compiling...")
        compileLibrary()
    # get location of current file
    path = os.path.abspath(os.path.join(
        os.path.dirname(__file__),)) + "/go/compiled/library.so"
    library = ctypes.cdll.LoadLibrary(path)
    return library



import json as toJson


class Response:
    def __init__(self, payload):
        self.status_code = payload.get("statusCode", 0)
        self.text = payload.get("body", "")
        self.cookies = payload.get("cookies", {})
        self.headers = payload.get("headers", {})
        self.url = payload.get("url", "")

    def json(self):
        try:
            return toJson.loads(self.text)
        except Exception as exception:
            raise exception

    def __str__(self):
        return str(self.status_code)



import ctypes


def pull_from_mem(bytes_) -> str:
    out = ctypes.string_at(bytes_)
    return out.decode('utf-8')


def check_error(json_):
    if json_.get("Error", False):
        raise Exception(json_['Error'])
    return True



from setuptools import setup, find_packages


setup(
    name='chromeRequests',
    version='0.2.3',
    author="Miro",
    license='MIT',
    packages=find_packages(),
    install_requires=[
        'scikit-learn',
    ],
    # include go folder
    package_data={
        'chromeRequests': ['go/compiled/library.so'],
    },
    include_package_data=True,
)
